!
! Version 30.04.2002 - H. Forbert   harald.forbert@theochem.ruhr-uni-bochum.de
!
! Changes:
! 29.01.2005 - pass filename as variable size string.   A. Kohlmeyer.
! 04.06.2005 - skip over << NEW DATA >> lines.          A. Kohlmeyer.          ! 30.06.2008 - remove residual average of data          A. Kohlmeyer.
!
! $Id: read_data.f90,v 1.1 2008/06/30 20:48:31 akohlmey Exp akohlmey $
!
MODULE read_data 

  USE kinds, ONLY : dbl
  USE string_utils, ONLY : xstring

  IMPLICIT NONE

  PRIVATE
  INTEGER iunit
  PARAMETER (iunit=10)
  PUBLIC :: check_file, read_file

CONTAINS

!! counts number of lines in file 'filename' to read
!! the format is:
!! i a2 a3 a4 a5 a6 a7
!! i : some integer index
!! a2,a3,a4 : real's that get ignored
!! a5,a6,a7 : 3 reals - vector of data
!! 
!! lines containing << NEW DATA >> as created by cpmd
!! on continuation of a previous run will be skipped.

SUBROUTINE check_file(filename, length, avg)

  IMPLICIT NONE

  CHARACTER (len=*), INTENT(IN) :: filename
  INTEGER, INTENT(OUT)    :: length
  REAL (dbl),INTENT(OUT)  :: avg(:)

  INTEGER :: n, l, i, ios, ia, ie
  CHARACTER(len=255) :: line
  REAL (dbl) :: a2,a3,a4,a5,a6,a7

  avg=0.0_dbl
  CALL xstring(filename,ia,ie)
  OPEN (UNIT = iunit, FILE = filename(ia:ie), STATUS = "OLD", ACTION = "READ", &
    IOSTAT = ios )
  IF (ios /= 0) THEN
     PRINT *,"Error opening file ",filename
     STOP
  END IF

  n = 0
  l = 0
  DO 
     READ (iunit,'(A)',IOSTAT=ios) line
     IF (ios /=0) EXIT
     IF (INDEX(line,'NEW DATA') == 0) THEN
        READ (line,*,IOSTAT=ios) i,a2,a3,a4,a5,a6,a7
        avg(1) = avg(1)+a5
        avg(2) = avg(2)+a6
        avg(3) = avg(3)+a7
        n = n + 1
        IF (ios /= 0) THEN
           PRINT*,'Error in data format at line', n+l
           STOP 
        ENDIF
     ELSE
        ! count skipped line
        l = l + 1
     ENDIF
  END DO

  CLOSE(UNIT=iunit, IOSTAT=ios)
  IF (ios /= 0) THEN
     PRINT *,"Error closing file ", filename
     STOP
  END IF

  print*, 'length of file: ', n, 'lines data,',l,'lines skipped.'
  avg(:) = avg(:)/dble(n)
  print*, 'residual average of data:',avg(1:3)
  length = n

END SUBROUTINE check_file

!! read a file in the same format as 'check_file'
!! input array will hold the data. The array must be
!! dimensioned sufficiently large in each dimension.

SUBROUTINE read_file(input, filename, nn, offs)

  IMPLICIT NONE

  REAL(dbl), DIMENSION ( : , : ), INTENT(INOUT)  :: input
  CHARACTER (len=*), INTENT(IN) :: filename
  INTEGER,   INTENT(IN) :: nn
  REAL(dbl), INTENT(IN) :: offs(:)

  INTEGER :: ix, i, n, ios, ia, ie
  CHARACTER(len=255) :: line
  REAL (dbl) :: a2,a3,a4

  CALL xstring(filename,ia,ie)
  OPEN (UNIT = iunit, FILE = filename(ia:ie), STATUS = "OLD", &
       ACTION = "READ", IOSTAT = ios )
  IF (ios /= 0) THEN
     PRINT *,"Error opening file ",filename
     STOP
  END IF

  n = SIZE(input,1)
  IF (nn < n) n = nn

  DO i = 1, n
     READ (iunit,'(A)',IOSTAT=ios) line
     IF (ios /=0) EXIT
     IF (INDEX(line,'NEW DATA') == 0) THEN
        READ (line,*,IOSTAT=ios) ix,a2,a3,a4,input(i,1),input(i,2),input(i,3)
        input(i,:) = input(i,:) - offs(:)
        IF (ios /= 0) THEN
           PRINT *,"Error reading file ",filename
           STOP
        END IF
     ENDIF
  END DO

  CLOSE(UNIT=iunit, IOSTAT=ios)
  IF (ios /= 0) THEN
     PRINT *,"Error closing file ", filename
     STOP
  END IF

  RETURN

END SUBROUTINE read_file

END MODULE read_data
