!
! Version 04.06.2005 - A. Kohlmeyer  axel.kohlmeyer@theochem.ruhr-uni-bochum.de
!
! $Id: write_data.f90,v 1.1 2005/06/04 11:31:54 qmmm Exp $
!
MODULE write_data 

  USE kinds, ONLY : dbl
  USE string_utils, ONLY : xstring

  IMPLICIT NONE
  INTEGER, PARAMETER :: out_len=250
  PRIVATE 
  INTEGER, PARAMETER :: iunit=20
  PUBLIC :: write_file
  
CONTAINS

!! normalize and write out the final spectral densities.

SUBROUTINE write_file(ftrans, wtrans, filename, nn, bh, wave_fac)

  IMPLICIT NONE

  REAL(dbl), DIMENSION ( : ), INTENT(IN)  :: ftrans, wtrans
  CHARACTER(len=out_len) , INTENT(IN) :: filename
  INTEGER ,      INTENT(IN) :: nn
  REAL (dbl),    INTENT(IN) :: wave_fac, bh

  REAL (dbl) :: norm_fourier, norm_classic, norm_kubo, norm_harmonic
  REAL (dbl) :: norm_shofield
  
  INTEGER    :: i, ios, ia, ie
  REAL (dbl) :: f, e, t, c

  CALL xstring(filename,ia,ie)
  norm_fourier  = 0.0_dbl
  norm_classic  = 0.0_dbl
  norm_kubo     = 0.0_dbl
  norm_harmonic = 0.0_dbl 
  norm_shofield = 0.0_dbl
  DO i = 0, nn
     t = wtrans(1+i)
     f = ftrans(1+i)
     e = t*(1.0_dbl - exp(-bh*t))
     norm_fourier  = norm_fourier  + f
     norm_classic  = norm_classic  + f*e
     norm_kubo     = norm_kubo     + f*e/(1.0_dbl+exp(-bh*t))
     norm_harmonic = norm_harmonic + f*t**2
     norm_shofield = norm_shofield + f*e*exp(0.5_dbl*bh*t)
  END DO 
  c = 1.0_dbl
  norm_fourier  = 1.0_dbl/(norm_fourier *c)
  norm_classic  = 1.0_dbl/(norm_classic *c)
  norm_kubo     = 1.0_dbl/(norm_kubo    *c)
  norm_harmonic = 1.0_dbl/(norm_harmonic*c)
  norm_shofield = 1.0_dbl/(norm_shofield*c)

  OPEN (UNIT = iunit, FILE = TRIM(filename(ia:ie)), STATUS = 'UNKNOWN', IOSTAT=ios)
  IF (ios /= 0) THEN
     PRINT *,'Failed to open output file!'
     STOP
  END IF

  DO i = 0, nn
     t = wtrans(1+i)
     f = ftrans(1+i)
     e = t*(1.0_dbl - exp(-bh*t))
     WRITE (iunit,'(1X,6ES13.5)',IOSTAT=ios) wave_fac*t,norm_fourier*f, &
        norm_classic *f*e, norm_kubo*f*e/(1.0_dbl+exp(-bh*t)), &
        norm_harmonic*f*t**2, &
        norm_shofield*f*e*exp(0.5_dbl*bh*t)
  END DO 

  CLOSE(iunit, IOSTAT=ios)

  RETURN

END SUBROUTINE write_file

END MODULE write_data
